from django.contrib.auth.models import AbstractBaseUser
from django.db import models
from django.utils import timezone
from django.core.validators import MinLengthValidator

from .managers import CustomUserManager


class User(AbstractBaseUser):
    login = models.CharField(max_length=16, validators=[MinLengthValidator(4)], verbose_name="Логин", unique=True)
    password = models.CharField(verbose_name="Пароль", max_length=128)
    last_login = models.DateTimeField(verbose_name="Последний вход", blank=True, null=True)
    is_superuser = models.BooleanField(default=False, verbose_name="Администратор")
    registered = models.DateTimeField(default=timezone.now, editable=False, verbose_name="Время регистрации")

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.is_staff = self.is_superuser

    last_password_change = models.DateTimeField(default=timezone.now, verbose_name="Последняя смена пароля")

    def set_password(self, raw_password):
        super().set_password(raw_password)
        self.last_password_change = timezone.now()

    USERNAME_FIELD = "login"
    REQUIRED_FIELDS = []

    objects = CustomUserManager()

    def __str__(self):
        return self.login

    class Meta:
        default_permissions = ()

    def has_perm(self, perm, obj=None):
        return self.is_superuser

    def has_module_perms(self, package_name):
        return self.is_superuser
